#################################################
#      COVID-19 SERIES FUNCTIONS        #
#################################################

################ Log-CPO #####################

Log_CPO <- function(param_chain){
     
      #param_mu=param[1:(length(param)-2)]
      #param_sig=param[(length(param)-1):length(param)]

      single.likelihood=c()
      single.log.likelihood=c()
      CPOq=c(0)
      CPOi=c(0)
      Q=c(dim(param_chain))

      for(q in 1:Q[1]){
          param_q=param_chain[q,]
          sig = sqrt(funcao_sig(param_q))
          med = funcao_mu(param_q)
          pred = med[,1]
          single.likelihood = dnorm(y, mean = pred, sd = sig, log = F)
          single.log.likelihood = dnorm(y, mean = pred, sd = sig, log = T)
          CPOq=CPOq+1/single.likelihood
        }
        CPOi=Q[1]/CPOq
        LCPOi=log(CPOi)

return(LCPOi)
}


Inf_points <- function(param_chain){
      
      #param_mu=param[1:(length(param)-2)]
      #param_sig=param[(length(param)-1):length(param)]

      single.log.likelihood_q=c(0)
      single.log.likelihood_i=c(0)
      Q=c(dim(param_chain))

      for(q in 1:Q[1]){
          param_q=param_chain[q,]
          sig = sqrt(funcao_sig(param_q))
          med = funcao_mu(param_q)
          pred = med[,1]
          single.log.likelihood_i= dnorm(y, mean = pred, sd = sig, log = T)
          single.log.likelihood_q = single.log.likelihood_q+single.log.likelihood_i
        }

      single.log.likelihood_i=single.log.likelihood_q/Q[1]

      LogCPOi=Log_CPO(param_chain)
  
      KLi=-LogCPOi+single.log.likelihood_i
      
      INFOi=0.5*(1+sqrt(1-exp(-2*KLi)))

return(INFOi)
}


Bay_Inf_Criterion <- function(param){
     
      k_param=length(param)
      nobs=length(y)

      med= funcao_mu(param)
      pred = med[,1]
      sig = sqrt(funcao_sig(param))
 

      single.log.likelihood = dnorm(y, mean = pred, sd = sig, log = T)
      
      BIC=k_param*log(nobs)-2*sum(single.log.likelihood)

return(BIC)
}



############## Likelihood  #####################

likelihood <- function(param){

param_mu=param[1:(length(param)-2)]
param_sig=param[(length(param)-1):length(param)]

sig = sqrt(funcao_sig(param))

med= funcao_mu(param)
pred = med[,1]

log.likelihood = sum(dnorm(y, mean = pred, sd = sig, log = T))
return(log.likelihood)
}

#############  Prior #########################

# Prior distribution

prior <- function(param){

  par.theta=param[1:(length(param)-2)]

  par.alpha=c(log(param[(length(param)-1)]),
                  log(param[length(param)]/(1-param[length(param)])))

  par.prior=c(par.theta,par.alpha)

  prior.mean=c(startvalue[1:(length(param)-2)],
                  log(startvalue[(length(param)-1)]),
                  log(startvalue[length(param)]/(1-startvalue[length(param)])))

  theta_prior = sum(dnorm(par.prior, mean =prior.mean, sd=sd.prior, log = T))

return(theta_prior)
}

############ Posterior #######################

posterior <- function(param){
return (likelihood(param) + prior(param))
}

######## Metropolis algorithm ################

proposalfunction <- function(param){

   proposal.mean=c(startvalue[1:(length(param)-2)],
                  log(startvalue[(length(param)-1)]),
                  log(startvalue[length(param)]/(1-startvalue[length(param)])))

  #  proposal.mean=c(param[1:(length(param)-2)],
  #                 log(param[(length(param)-1)]),
  #                 log(param[length(param)]/(1-param[length(param)])))



   proposal.par=rnorm(length(param),mean = proposal.mean, sd= sd.proposal)

   proposal.theta=proposal.par[1:(length(param)-2)]

   proposal.alpha=c(exp( proposal.par[(length(param)-1)] ), 
                    exp(proposal.par[length(param)])/(1+exp(proposal.par[length(param)])))

return(c(proposal.theta,proposal.alpha))
}

run_metropolis_MCMC <- function(startvalue, iterations){
chain = array(dim = c(iterations+1,length(startvalue)))
chain[1,] = startvalue

for (i in 1:iterations){
    proposal = proposalfunction(chain[i,])

    probab = exp(posterior(proposal) - posterior(chain[i,]))

    if(runif(1) < probab){
       chain[i+1,] = proposal
       }
		else{
       	chain[i+1,] = chain[i,]
       	}
}
return(chain)
}


######## Models  ##########

# Function for Variance

funcao_sig=function(param){
    
    param_mu=param[1:(length(param)-2)]
    param_sig=param[(length(param)-1):length(param)]

    nobs=length(y)

    med=funcao_mu(theta)
    med_y=med[,1]

    W2=c()
    W2[1]=0
    W2[2:nobs]=(y[1:(nobs-1)]-med_y[1:(nobs-1)])^2  
    
    alpha1=param_sig[1]
    alpha2=param_sig[2]
    
    fun_sig=alpha1+alpha2*W2
 
return(fun_sig)
}

Z_model=function(theta_Z,fun_mut){

   Zch=c()
   if (p>0){
     nobs=length(y)
     Z=y-fun_mut
     Zch[1:p]=Z[1:p]
     Zch[(p+1):nobs]=0
     for (ires in (p+1):nobs){
          for (iord in 1:p){
              Zch[ires]=Zch[ires]+theta_Z[iord]*Z[ires-iord]
              }
      }
    }

return(Zch)
}

ARMA_ARCH=function(Z){

   # Modelo AR(p)
    fit_Z <-auto.arima(Z, d=0,max.p=10, max.q=0, allowdrift=F, allowmean=F, trace=F,ic = c("bic"))

    theta_Z=fit_Z$coef  
    p=length(theta_Z)

   # Modelo ARCH(1)
    W=c()
    W=fit_Z$residuals
    W2=W^2
    fit_W2 <- arima(W2, order = c(1, 0, 0))
    alpha=rev(abs(fit_W2$coef))

    param_Z=c(theta_Z,alpha,p)

return(param_Z)
}

# Functions for Mean

#1. Polinomial

Poli_mu=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta4=param[4]

   theta_Z=param[5:(length(param)-1)]
 
   fun_mut=c()
   Zch=c()

   fun_mut=(theta1+theta2*dia)/(1+theta3*dia+theta4*(dia^2))

   fun_mut[fun_mut<=0]=0

   Zch=Z_model(theta_Z,fun_mut)

   fun_mu=fun_mut+Zch

   fun_mu[fun_mu<=0]=0

return(cbind(fun_mu,fun_mut))
}

start.Poli_mu=function(param){
adjust_mu=nls(y~((theta1+theta2*dia)/(1+theta3*dia+theta4*dia^2)),
         start=list(theta1=param[1],theta2=param[2],theta3=param[3],theta4=param[4]),
         control=list(maxiter=5000,tol=10e-5,minFactor=10e-26))

   f_adjust_mu=summary(adjust_mu)

   theta1=f_adjust_mu$parameters[1,1]
   theta2=f_adjust_mu$parameters[2,1]
   theta3=f_adjust_mu$parameters[3,1]
   theta4=f_adjust_mu$parameters[4,1]

   fun_mut=c()
   Z=c()

   nobs=length(y)

   fun_mut=(theta1+theta2*dia)/(1+theta3*dia+theta4*(dia^2))

   fun_mut[fun_mut<=0]=0
   
   Z[1:nobs]=y[1:nobs]-fun_mut[1:nobs] 
  
   param_Z=ARMA_ARCH(Z)

   param_adjust=c(f_adjust_mu$parameters[,1],param_Z)

return(param_adjust)
}

#2. Log-Normal

LogNorm_mu=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:(length(param)-1)]

   fun_mut=c()
   Zch=c()
   fun_mut=(theta1/dia)*exp(-((log(dia)-theta2)^2)/theta3)

   fun_mut[fun_mut<=0]=0

   Zch=Z_model(theta_Z,fun_mut)

   fun_mu=fun_mut+Zch

   fun_mu[fun_mu<=0]=0

return(cbind(fun_mu,fun_mut))
}

start.LogNorm_mu=function(param){

adjust_mu=nls(y~((theta1/dia)*exp(-((log(dia)-theta2)^2)/theta3)),
              start=list(theta1=param[1],theta2=param[2],theta3=param[3]),
              control=list(maxiter=5000,tol=10e-5,minFactor=10e-26))

   f_adjust_mu=summary(adjust_mu)

   theta1=f_adjust_mu$parameters[1,1]
   theta2=f_adjust_mu$parameters[2,1]
   theta3=f_adjust_mu$parameters[3,1]

   fun_mut=c()
   Z=c()

   nobs=length(y)

   fun_mut=(theta1/dia)*exp(-((log(dia)-theta2)^2)/theta3)

   fun_mut[fun_mut<=0]=0
   
   Z[1:nobs]=y[1:nobs]-fun_mut[1:nobs] 
  
    param_Z=ARMA_ARCH(Z)

    param_adjust=c(f_adjust_mu$parameters[,1],param_Z)

return(param_adjust)
}

#3. Gaussian

Gauss_mu=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:(length(param)-1)]

   fun_mut=c()
   Zch=c()
 
   fun_mut=(theta1)*exp(-((dia-theta2)^2)/theta3)

   fun_mut[fun_mut<=0]=0

   Zch=Z_model(theta_Z,fun_mut)

   fun_mu=fun_mut+Zch

   fun_mu[fun_mu<=0]=0

return(cbind(fun_mu,fun_mut))
}

start.Gauss_mu=function(param){

adjust_mu=nls(y~((theta1)*exp(-((dia-theta2)^2)/theta3)),
              start=list(theta1=param[1],theta2=param[2],theta3=param[3]),
              control=list(maxiter=5000,tol=10e-5,minFactor=10e-26))

   f_adjust_mu=summary(adjust_mu)

   theta1=f_adjust_mu$parameters[1,1]
   theta2=f_adjust_mu$parameters[2,1]
   theta3=f_adjust_mu$parameters[3,1]

   fun_mut=c()
   Z=c()

   nobs=length(y)

   fun_mut=(theta1)*exp(-((dia-theta2)^2)/theta3)

   fun_mut[fun_mut<=0]=0
   
   Z[1:nobs]=y[1:nobs]-fun_mut[1:nobs] 
  
   param_Z=ARMA_ARCH(Z)

   param_adjust=c(f_adjust_mu$parameters[,1],param_Z)

return(param_adjust)
}


#4. Gamma

Gamma_mu=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:(length(param)-1)]

   fun_mut=c()
   Zch=c()

   fun_mut=(theta1)*(dia^theta2)*exp(-theta3*dia)

   fun_mut[fun_mut<=0]=0

   Zch=Z_model(theta_Z,fun_mut)

   fun_mu=fun_mut+Zch

   fun_mu[fun_mu<=0]=0

return(cbind(fun_mu,fun_mut))
}

start.Gamma_mu=function(param){

adjust_mu=nls(y~(theta1*(dia^theta2)*exp(-theta3*dia)),
              start=list(theta1=param[1],theta2=param[2],theta3=param[3]),
              control=list(maxiter=5000,tol=10e-5,minFactor=10e-26))

   f_adjust_mu=summary(adjust_mu)

   theta1=f_adjust_mu$parameters[1,1]
   theta2=f_adjust_mu$parameters[2,1]
   theta3=f_adjust_mu$parameters[3,1]

   fun_mut=c()
   Z=c()

   nobs=length(y)

   fun_mut=(theta1)*(dia^theta2)*exp(-theta3*dia)

   fun_mut[fun_mut<=0]=0
  
   Z[1:nobs]=y[1:nobs]-fun_mut[1:nobs] 
  
   param_Z=ARMA_ARCH(Z)

   param_adjust=c(f_adjust_mu$parameters[,1],param_Z)

return(param_adjust)
}

#5. Generalized Exponential

ExpGen_mu=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:(length(param)-1)]

   fun_mut=c()
   Zch=c()

   fun_mut=(theta1*(1-exp(-theta2*dia))^(theta3-1))*exp(-theta2*dia)

   fun_mut[fun_mut<=0]=0
   
   Zch=Z_model(theta_Z,fun_mut)

   fun_mu=fun_mut+Zch

   fun_mu[fun_mu<=0]=0

return(cbind(fun_mu,fun_mut))
}

start.ExpGen_mu=function(param){

adjust_mu=nls(y~((theta1*(1-exp(-theta2*dia))^(theta3-1))*exp(-theta2*dia)),
              start=list(theta1=param[1],theta2=param[2],theta3=param[3]),
              control=list(maxiter=5000,tol=10e-5,minFactor=10e-26))

   f_adjust_mu=summary(adjust_mu)

   theta1=f_adjust_mu$parameters[1,1]
   theta2=f_adjust_mu$parameters[2,1]
   theta3=f_adjust_mu$parameters[3,1]

   fun_mut=c()
   Z=c()

   nobs=length(y)

   fun_mut=(theta1*(1-exp(-theta2*dia))^(theta3-1))*exp(-theta2*dia)

   fun_mut[fun_mut<=0]=0
   
   Z[1:nobs]=y[1:nobs]-fun_mut[1:nobs] 
  
   param_Z=ARMA_ARCH(Z)

   param_adjust=c(f_adjust_mu$parameters[,1],param_Z)

return(param_adjust)

}


#6. Alpha

Alpha_mu=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:(length(param)-1)]
 
   fun_mut=c()
   Zch=c()

   fun_mut=(theta1/(dia^2))*exp(-((theta2/dia-theta3)^2)/2)

   fun_mut[fun_mut<=0]=0
   
   Zch=Z_model(theta_Z,fun_mut)

   fun_mu=fun_mut+Zch

   fun_mu[fun_mu<=0]=0

return(cbind(fun_mu,fun_mut))
}

start.Alpha_mu=function(param){

adjust_mu=nls(y~((theta1/(dia^2))*exp(-((theta2/dia-theta3)^2)/2)),
              start=list(theta1=param[1],theta2=param[2],theta3=param[3]),
              control=list(maxiter=5000,tol=10e-5,minFactor=10e-26))

   f_adjust_mu=summary(adjust_mu)

   theta1=f_adjust_mu$parameters[1,1]
   theta2=f_adjust_mu$parameters[2,1]
   theta3=f_adjust_mu$parameters[3,1]

   fun_mut=c()
   Z=c()

   nobs=length(y)

   fun_mut=(theta1/(dia^2))*exp(-((theta2/dia-theta3)^2)/2)

   fun_mut[fun_mut<=0]=0
   
   Z[1:nobs]=y[1:nobs]-fun_mut[1:nobs] 
  
   param_Z=ARMA_ARCH(Z)

   param_adjust=c(f_adjust_mu$parameters[,1],param_Z)

return(param_adjust)
}


#7. Logstico

Logistico_mu=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:(length(param)-1)]
 
   fun_mut=c()
   Zch=c()

   fun_mut=(theta1*exp(-theta3*dia))/((1+theta2*exp(-theta3*dia))^2)

   fun_mut[fun_mut<=0]=0

   Zch=Z_model(theta_Z,fun_mut)

   fun_mu=fun_mut+Zch

   fun_mu[fun_mu<=0]=0

return(cbind(fun_mu,fun_mut))
}

start.Logistico_mu=function(param){

adjust_mu=nls(y~((theta1*exp(-theta3*dia))/((1+theta2*exp(-theta3*dia))^2)),
              start=list(theta1=param[1],theta2=param[2],theta3=param[3]),
              control=list(maxiter=5000,tol=10e-5,minFactor=10e-26))

   f_adjust_mu=summary(adjust_mu)

   theta1=f_adjust_mu$parameters[1,1]
   theta2=f_adjust_mu$parameters[2,1]
   theta3=f_adjust_mu$parameters[3,1]

   fun_mut=c()
   Z=c()

   nobs=length(y)

   fun_mut=(theta1*exp(-theta3*dia))/((1+theta2*exp(-theta3*dia))^2)

   fun_mut[fun_mut<=0]=0
   
   Z[1:nobs]=y[1:nobs]-fun_mut[1:nobs] 
  
    param_Z=ARMA_ARCH(Z)

    param_adjust=c(f_adjust_mu$parameters[,1],param_Z)

return(param_adjust)
}


###---------------Forecast -------------------###

#1. Forecast - Polinomiall

Poli_mu_prev=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta4=param[4]
   theta_Z=param[5:length(param)]

   nobs=length(y)
 
   fun_mut_prev=c()
   Z_ch=c()
   Z_prev=c()

   fun_mut_prev=(theta1+theta2*dia_prev)/(1+theta3*dia_prev+theta4*(dia_prev^2))

   fun_mut_prev[fun_mut_prev<=0]=0
   
    nprev=length(dia_prev)
    Z_prev[1:nprev]=0
    if(p>0){

      Z_ch=Z

      for (iprev in 1:nprev){
          for (iord in 1:p){
               Z_prev[iprev]=Z_prev[iprev]+theta_Z[iord]*Z_ch[nobs+iprev-iord]
               }
       Z_ch[nobs+iprev]=Z_prev[iprev]
       if (length(Y_Obs)>0)
         {Z_ch[nobs+iprev]= Y_Obs[iprev]/scale-fun_mut_prev[iprev]} 
       }
     }
     fun_mu_prev=fun_mut_prev+Z_prev

     fun_mu_prev[fun_mu_prev<=0]=0

return(fun_mu_prev)
}


#2. Forecast - Log-Normal

LogNorm_mu_prev=function(param){

    theta1=param[1]
    theta2=param[2]
    theta3=param[3]
    theta_Z=param[4:length(param)]
 
    nobs=length(y)
 
    fun_mut_prev=c()
    Z_ch=c()
    Z_prev=c()

    fun_mut_prev=(theta1/dia_prev)*exp(-((log(dia_prev)-theta2)^2)/theta3)
 
    fun_mut_prev[fun_mut_prev<=0]=0
   
    nprev=length(dia_prev)
    Z_prev[1:nprev]=0
    if(p>0){
      Z_ch=Z

      for (iprev in 1:nprev){
          for (iord in 1:p){
               Z_prev[iprev]=Z_prev[iprev]+theta_Z[iord]*Z_ch[nobs+iprev-iord]
               }
       Z_ch[nobs+iprev]=Z_prev[iprev]
       if (length(Y_Obs)>0)
         {Z_ch[nobs+iprev]= Y_Obs[iprev]/scale-fun_mut_prev[iprev]} 
       }
     }
     fun_mu_prev=fun_mut_prev+Z_prev

     fun_mu_prev[fun_mu_prev<=0]=0

return(fun_mu_prev)
}


#3. Forecast - Gaussian

Gauss_mu_prev=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:length(param)]

   nobs=length(y)
 
   fun_mut_prev=c()
   Z_ch=c()
   Z_prev=c()

   fun_mut_prev=(theta1)*exp(-((dia_prev-theta2)^2)/theta3)

   fun_mut_prev[fun_mut_prev<=0]=0
   
    nprev=length(dia_prev)
    Z_prev[1:nprev]=0

    if(p>0){

      Z_ch=Z

      for (iprev in 1:nprev){
          for (iord in 1:p){
               Z_prev[iprev]=Z_prev[iprev]+theta_Z[iord]*Z_ch[nobs+iprev-iord]
               }
       Z_ch[nobs+iprev]=Z_prev[iprev]
       if (length(Y_Obs)>0)
         {Z_ch[nobs+iprev]= Y_Obs[iprev]/scale-fun_mut_prev[iprev]} 
       }
     }
     fun_mu_prev=fun_mut_prev+Z_prev

     fun_mu_prev[fun_mu_prev<=0]=0


return(fun_mu_prev)
}

#4. Forecast - Gamma

Gamma_mu_prev=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:length(param)]

   nobs=length(y)
 
   fun_mut_prev=c()
   Z_ch=c()
   Z_prev=c()

   fun_mut_prev=(theta1)*(dia_prev^theta2)*exp(-theta3*dia_prev)

   fun_mut_prev[fun_mut_prev<=0]=0
   
   nprev=length(dia_prev)
   Z_prev[1:nprev]=0

   if(p>0){

     Z_ch=Z

     for (iprev in 1:nprev){
         for (iord in 1:p){
              Z_prev[iprev]=Z_prev[iprev]+theta_Z[iord]*Z_ch[nobs+iprev-iord]
              }
      Z_ch[nobs+iprev]=Z_prev[iprev]
      if (length(Y_Obs)>0)
         {Z_ch[nobs+iprev]= Y_Obs[iprev]/scale-fun_mut_prev[iprev]} 
      }
     }
     fun_mu_prev=fun_mut_prev+Z_prev

     fun_mu_prev[fun_mu_prev<=0]=0

return(fun_mu_prev)
}

#5. Forecast - Exponencial Generalizada

ExpGen_mu_prev=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:length(param)]

   nobs=length(y)
 
   fun_mut_prev=c()
   Z_ch=c()
   Z_prev=c()

   fun_mut_prev=(theta1*(1-exp(-theta2*dia_prev))^(theta3-1))*exp(-theta2*dia_prev)

   fun_mut_prev[fun_mut_prev<=0]=0
   
    nprev=length(dia_prev)
    Z_prev[1:nprev]=0

    if(p>0){

      Z_ch=Z

      for (iprev in 1:nprev){
          for (iord in 1:p){
               Z_prev[iprev]=Z_prev[iprev]+theta_Z[iord]*Z_ch[nobs+iprev-iord]
               }
       Z_ch[nobs+iprev]=Z_prev[iprev]
       if (length(Y_Obs)>0)
         {Z_ch[nobs+iprev]= Y_Obs[iprev]/scale-fun_mut_prev[iprev]} 
       }
     }
     fun_mu_prev=fun_mut_prev+Z_prev

     fun_mu_prev[fun_mu_prev<=0]=0

return(fun_mu_prev)
}

#6. Forecast - Alpha

Alpha_mu_prev=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:length(param)]

   nobs=length(y)
 
   fun_mut_prev=c()
   Z_ch=c()
   Z_prev=c()

   fun_mut_prev=((theta1/(dia_prev^2))*exp(-((theta2/dia_prev-theta3)^2)/2))

   fun_mut_prev[fun_mut_prev<=0]=0
   
    nprev=length(dia_prev)
    Z_prev[1:nprev]=0

    if(p>0){

      Z_ch=Z

      for (iprev in 1:nprev){
          for (iord in 1:p){
               Z_prev[iprev]=Z_prev[iprev]+theta_Z[iord]*Z_ch[nobs+iprev-iord]
               }
      Z_ch[nobs+iprev]= Z_prev[iprev]
      if (length(Y_Obs)>0)
         {Z_ch[nobs+iprev]= Y_Obs[iprev]/scale-fun_mut_prev[iprev]} 
      }
     }
     fun_mu_prev=fun_mut_prev+Z_prev

     fun_mu_prev[fun_mu_prev<=0]=0
  
return(fun_mu_prev)
}

#7.Forecast - Logstico

Logistico_mu_prev=function(param){

   theta1=param[1]
   theta2=param[2]
   theta3=param[3]
   theta_Z=param[4:length(param)]
 
    nobs=length(y)
 
    fun_mut_prev=c()
    Z_ch=c()
    Z_prev=c()

    fun_mut_prev=(theta1*exp(-theta3*dia_prev))/((1+theta2*exp(-theta3*dia_prev))^2)

    fun_mut_prev[fun_mut_prev<=0]=0
   
    nprev=length(dia_prev)
    Z_prev[1:nprev]=0

    if(p>0){

      Z_ch=Z

      for (iprev in 1:nprev){
          for (iord in 1:p){
               Z_prev[iprev]=Z_prev[iprev]+theta_Z[iord]*Z_ch[nobs+iprev-iord]
               }
       Z_ch[nobs+iprev]=Z_prev[iprev]
       if (length(Y_Obs)>0)
         {Z_ch[nobs+iprev]= Y_Obs[iprev]/scale-fun_mut_prev[iprev]} 
       }
     }
     fun_mu_prev=fun_mut_prev+Z_prev

     fun_mu_prev[fun_mu_prev<=0]=0

return(fun_mu_prev)
}


funcao_sig_prev=function(param){
    
    param_mu=param[1:(length(param)-2)]
    param_sig=param[(length(param)-1):length(param)]

    nobs=length(y)
    nprev=length(dia_prev)
    y_obs=Y_Obs/scale

    W2_nobs=(y[nobs]-med_y[nobs])^2        
 
    alpha1=param_sig[1]
    alpha2=param_sig[2]
    var_prev=c()
    var_prev[1]=alpha1+alpha2*W2_nobs

    for (j in 2 : nprev){
         var_prev[j]=alpha1+alpha2*var_prev[j-1]
         if (length(Y_Obs)>0){
             var_prev[j]=alpha1+alpha2*(y_obs[j-1]-med_prev[j-1])^2}
          }

    fun_sig_prev=var_prev

return(fun_sig_prev)
}




