

############### PROGRAM: MH_MCMC_R-COVID19 #####################
#
# Packages:
# install.packages(tseries)
# install.packages("forecast")
# ------- 

# Functions:
# source('Funcoes_COVID19.r')
# source('archtest.r')
# ---------

# STEPS
#1. Reading Data Set: Reading data from each country with initial conditions for the parameters of each function.
#2. Choose a Model: For each model chosen, identify the average function and the order of the AR model (p)
#3. Initial conditions 
#4. Model Preview 
#   4.1 Shows the model with the initial conditions:
#   4.2 Z(t) ~ AR(p): Shows the order of the AR (p) model selected with the BIC criterion for the initial conditions;
#   4.3 Z(t) ~ ARCH(1): Apply the LM test for ARCH(1) model for Z (t) process 
#5. MH-MCMC
#   5.1 Acceptance Rate: Set acceptance rate for min 40% MCMC acceptance
#   5.2 Selecting chain: Apply the burning period: and selected chain.
#   5.3 Functions:
#----- Geweke criterion --- For mu(t)+AR(p)+ARCH(1)----#
#------Bayesian estimates -----------------------------#
#----- CPO and BIC ------------------------------------#
#------Influential points  ----------------------------# 
#------Graphic: Mean and IC ---------------------------#
#------Graphic: Z(t) Residual analysis ----------------#
#------Forecast----------------------------------------# 
#------Graphic: Forecast da Mean and CI ---------------#
#------Peak forecast ----------------------------------#
#----- Forecast the end of the pandemic ---------------#
#----  Forecast Error ---------------------------------#

#####################################################################

rm(list=ls(all=TRUE))
setwd('C:\\COVID19\\Programa')
source('Funcoes_COVID19.r')
source('archtest.r')

#install.packages(tseries)
#install.packages("forecast")
library(tseries)
library(forecast)
library(MASS)
library(coda)
library(aTSA)

#----------------------  Read Data ----------------------------------------------
#             May 10   May 20   May 30  Jun 10  Jun 20   jun 30   jul 31   Aug 20
# -------------------------------------------------------------------------------
# China       132      142      152      162      172     182      213     233
# USA         111      121      131      141      151     161      192     212
# Italy       101      111      121      131      141     151      182     202
# Spain       100      110      120      130      140     150      181     201
# Brazil       81       91      101      111      121     131      162     182
# India       101      111      121      131      141     151      182     202
#--------------------------------------------------------------------------------


#------------------- Reading Data Set ---------------------------
# China

Country="China"
COVID19 <- read.table('Dados_COVID19_ChinaB.txt', header = T)
PopChina=1411124099
scale=PopChina/10^8
n_cn=213
x <- COVID19[1:n_cn,1]/scale
y <- COVID19[1:n_cn,2]/scale
#theta0=c(0.2622, 0.0025, -0.0406, 0.00042895)                 # Rat. Function
#theta0=c( 414.005687690850, 3.884999545037, 0.095890827753)   # Log-Normal
#theta0=c(8.755799063984, 47.892885133164, 209.085735385082)   # Gaussiano
#theta0=c(1.4*10^(-13), 11.17175870910450,   0.24349645520474) # Gamma
#theta0=c(2276.37534151151, 0.10046664661, 97.20472832697)     # Generalized Exp.
#theta0=c(19941.7463296728, 215.3356845844, 4.3344828829)      # Alpha
#theta0=c(62018.54, 1720.472, 0.1553762 )                      # Logistico, m=1;
###
dia=1:length(y)
#---------------------------------------------------------

#---------------------------------------------------------
# USA

Country="USA"
COVID19 <- read.table('Dados_COVID19_USAB.txt', header = T)
PopUS=333761149
scale=PopUS/10^6
n_usa=192
x <- COVID19[1:n_usa,1]/scale
y <- COVID19[1:n_usa,2]/scale
theta0=c(-0.37014033740408, 0.00733047026646, -0.02224627990514, 0.00012849635517) # Rat. Function
#theta0=c(670.433940033012, 4.552360903586,  0.058860483561)                       # Log-Normal
#theta0=c(7.248286192938,  93.353962889864,  448.020144150328)                     # Gaussiano
#theta0=c(5.556493e-54, 35.32274, 0.3817905)                                       # Gamma
#theta0=c(647.85211408,   0.03776264,  39.52055966 )                               # Generalized Exp.
#theta0= c(63016.0322575116, 489.9432847912, 5.0155487659)                         # Alpha 
#theta0=theta0=c(7.8663e+003,349.1408, 0.0522)                                     # Logistico # m=1
###
dia=1:length(y)
#---------------------------------------------------------

#----------------------------------------------------------
# Italy

Country="Italy"
COVID19 <- read.table('Dados_COVID19_ItalyB.txt', header = T)
PopItaly=60020970
scale=PopItaly/10^6
n_it=182
x <- COVID19[1:n_it,1]/scale
y <- COVID19[1:n_it,2]/scale
#theta0=c(-0.70115797292251, 0.01276063072512, -0.02182773266515, 0.00012331190498)  # Rat. Function
#theta0=c(811.9435781, 4.2205555, 0.1437534 )                                        # Log-Normal
#theta0=c(12.187237131460, 97.640783885915, 591.302923454607)                        # Gaussiano
#theta0=c(9.700772e-19, 1.389671e+01, 2.157766e-01)                                  # Gamma
#theta0=c(1522.65827025246, 0.06102579219, 45.08483914521)                           # Generalized Exp.
theta0=c(54329.6000941139, 220.5561503179, 3.0728481306)                             # Alpha
#theta0=c(2.180641e+04, 4.370823e+02, 9.188276e-02)                                  # Logistico # m=1
###
dia=1:length(y)
#-----------------------------------------------------------

#----------------------------------------------------------
# Spain

Country="Spain"
COVID19 <- read.table('Dados_COVID19_SpainB.txt', header = T)
PopSpain=45680787
scale=PopSpain/10^6
n_es=181
x <- COVID19[1:n_es,1]/scale
y <- COVID19[1:n_es,2]/scale
#theta0=c(-1.079810599,  0.030586857, -0.032004765,  0.000267125)  # Rat. Function
#theta0=c(1178.0391526, 4.2190563, 0.0821118 )                     # Log-Normal.
#theta0=c(17.41646,  67.24294, 359.51315)                          # Gaussiano
#theta0=c(6.022378e-33, 2.417015e+01, 3.667946e-01)                # Gamma
#theta0=c(8723.14483850996, 0.08100617306, 177.06986410402)        # Generalized Exp.
theta0=c(79197.4665296395, 314.7665757429, 4.5509849207)           # Alpha
#theta0=c(1.939334e+05, 2.712394e+03, 1.182679e-01 )               # Logistico # m=1
###
dia=1:length(y)
#---------------------------------------------------------

#-----------------------------------------------------------
# Brazil

Country="Brazil"
COVID19 <- read.table('Dados_COVID19_Brazil2.txt', header = T)
PopBrazil=217256782
scale=PopBrazil/10^6
n_br=182
x <- COVID19[1:n_br,1]/scale
y <- COVID19[1:n_br,2]/scale
#theta0=c( -0.08499976158936, 0.00533238605157, -0.01867792377061, 0.00009622029652) # Rat. Function
#theta0=c(618.8971531, 4.9219351, 0.3007477 )                                        # Log-Normal
#theta0=c(4.22943065898, 105.46391799792, 1516.32396307943)                          # Gaussiano
#theta0=c(1.459184e-09, 5.668812, 4.331828e-02 )                                     # Gamma
#theta0=c(159.02399715,  0.02152875, 12.54875693 )                                   # Generalized Exp.
theta0= c(771747.95363,    126.68969,     -1.13883)                                  # Alpha  
#theta0=c(15924.8256068086, 818.9974393847, 0.0645846835)                            # Logistico #m=1
###
dia=1:length(y)
#------------------------------------------------------------------

#-----------------------------------------------------------
# India

Country="India"
COVID19 <- read.table('Dados_COVID19_IndiaB.txt', header = T)
PopIndia=1394618848
scale=PopIndia/10^8
n_in=182
x <- COVID19[1:n_in,1]/scale
y <- COVID19[1:n_in,2]/scale
#theta0=c(-0.55494700724747,  0.01641023613009,  -0.01284104818243,  0.00004449210121) # Rat. Function
theta0=c(3.290014e+04, 5.997861, 0.5170383)                                            # Log-Normal
#theta0=c(41.21793874003, 188.11132187751,  4347.87634080824)                          # Gaussian
#theta0=c( 0.00000000050078, 5.45659115886636, 0.01625654735372)                       # Gamma
#theta0=c(831.877045622061, 0.009499280142, 8.768011478068)                            # Generalized Exp.
#theta0=theta0=c(7.85973257317210e+010, 153.5851, -3.7769)                             # Alpha
#theta0=c(1.67171991322224e+05, 1663.467408212, 0.048665106)                           #Logstico # m=1 
###
dia=1:length(y)
#------------------------------------------------------------------

##-------------- Choose a Model--------------##

#--------  Rational Functions ----#
mod_name='RatFunc'
start=start.Poli_mu(theta0)
theta=start[1:(length(start)-1)]
p=start[length(start)]
med=Poli_mu(theta)
med_y=med[,1]
med_mut=med_y
if (p>0)med_mut=med[,2]
funcao_mu=Poli_mu
#---------------------------------------------------

#---------  Alpha ------------#
mod_name='Alpha'
start=start.Alpha_mu(theta0)
theta=start[1:(length(start)-1)]
p=start[length(start)]
med=Alpha_mu(theta)
med_y=med[,1]
med_mut=med_y
if (p>0)med_mut=med[,2]
funcao_mu=Alpha_mu
#---------------------------------------------------

#---------  Logistico --------#
mod_name='Logistico'
start=start.Logistico_mu(theta0)
theta=start[1:(length(start)-1)]
p=start[length(start)]
med=Logistico_mu(theta)
med_y=med[,1]
med_mut=med_y
if (p>0)med_mut=med[,2]
funcao_mu=Logistico_mu
#---------------------------------------------------

#---------- Log-Normal -------#
mod_name='LogNormal'
start=start.LogNorm_mu(theta0)
theta=start[1:(length(start)-1)]
p=start[length(start)]
med=LogNorm_mu(theta)
med_y=med[,1]
med_mut=med_y
if (p>0)med_mut=med[,2]
funcao_mu=LogNorm_mu
#---------------------------------------------------

#---------- Gaussiano  -------#
mod_name='Gaussiano'
start=start.Gauss_mu(theta0)
theta=start[1:(length(start)-1)]
p=start[length(start)]
med=Gauss_mu(theta)
med_y=med[,1]
med_mut=med_y
if (p>0)med_mut=med[,2]
funcao_mu=Gauss_mu
#---------------------------------------------------

#---------- Gamma  -----------#
mod_name='Gamma'
start=start.Gamma_mu(theta0)
theta=start[1:(length(start)-1)]
p=start[length(start)]
med=Gamma_mu(theta)
med_y=med[,1]
med_mut=med_y
if (p>0)med_mut=med[,2]
funcao_mu=Gamma_mu
#---------------------------------------------------

#---- Generalized Exponential ------#
mod_name='ExpGen'
start=start.ExpGen_mu(theta0)
theta=start[1:(length(start)-1)]
p=start[length(start)]
med=ExpGen_mu(theta)
med_y=med[,1]
med_mut=med_y
if (p>0)med_mut=med[,2]
funcao_mu=ExpGen_mu
#---------------------------------------------------

#-------------- Initial conditions ----------------#

#------- Model Preview ----------#
plot(dia,y,ylim=c(0,max(y,med_y)),pch=8)
lines(dia,med_y,col=2,lwd=2)
#---------------------------------------------------#

#-------- Z(t) ~ AR(p)-----------# 
Z=y-med_mut
auto.arima(Z, d=0,max.p=10, max.q=0, allowdrift=F, allowmean=F, trace=T,ic = c("bic"))

#-------- Z(t) ~ ARCH(1)---------#
# ARCH Test 
archtest(ts=Z, lag=1)

#-- Acceptance rate -------------#
acceptance.rate=min(abs(theta))/10
acceptance.rate=10^(-3)
# ------------- MH-MCMC ----------------------------#
startvalue = c(theta)
sd.proposal=rep(acceptance.rate,length(startvalue))
sd.prior=rep(10^(-1),length(startvalue))
chain = run_metropolis_MCMC(startvalue, 50000)
burnIn = 15000
acceptance = 1-mean(duplicated(chain[-(1:burnIn),]))
acceptance

#-------------  Selecting chain --------------------#
sel_chain=chain[-(1:burnIn),]
theta_bayes=sel_chain[seq(1,nrow(sel_chain),by=10),]
#---------------------------------------------------#

#---- Read save chain -----------------------------------------#
#theta_bayes=read.table('Theta_ RatFunc _ USA .txt', header = F)
#--------------------------------------------------------------#

#----- Geweke --- For mu(t)+AR(p)+ARCH(1)-----#
gw=apply(theta_bayes,2,geweke.diag) 
GW=cbind(gw[[1]][1])
for (igw in 2:length(theta)){
    GW=cbind(GW,gw[[igw]][1])
    }
GW
#---------------------------------------------#

#-------------  Bayesian estimates -----------#
theta_ch_bayes=apply(theta_bayes,2,mean)
sd_theta_bayes=apply(theta_bayes,2,sd)
med_theta_bayes=apply(theta_bayes,2,median)
IC_theta_bayes=apply(theta_bayes,2,quantile,prob=c(0.025,0.975))

Est_bayes=cbind(theta_ch_bayes,med_theta_bayes,sd_theta_bayes,t(IC_theta_bayes))
Est_bayes


#--- Mean ------------------------------#

med=funcao_mu(theta_ch_bayes)
med_y=med[,1]
med_mut=med_y
if (p>0)med_mut=med[,2]
var_y=funcao_sig(theta_ch_bayes)

#----- Z(t) ~ AR(p) ---------------------#
Z=y-med_mut
Z2=Z^2
auto.arima(Z, d=0,max.p=10, max.q=0, allowdrift=F, allowmean=F, trace=T,ic = c("bic"))

#--- Z(t) ~ ARCH(1) -------#
archtest(ts=Z, lag=1)

#--- W ~ ARCH(1) ----------#
# ARCH Test 
W=y-med_y
W2=W^2
Ws=(W-mean(W))/sqrt(var_y)
archtest(ts=W, lag=1)

#--------- CPO and BIC ------------------#
LogCPOi=Log_CPO(sel_chain)
LogCPO=sum(LogCPOi)
LogCPO
BIC=Bay_Inf_Criterion(theta_ch_bayes)
BIC
#----------------------------------------#

#------ Influential points  -------------#      
RHOi=Inf_points(sel_chain)
#outlier_day=c(dia[which(RHOi>quantile(RHOi, probs =0.99))])
outlier_day=c(dia[which(RHOi>0.8)])
outlier_data=rbind(outlier_day, y[outlier_day], scale*y[outlier_day])
outlier_data
#----------------------------------------#

#--- IC 95% -----------#  
ICy025 =med_y-1.96*sqrt(var_y)
ICy025[ICy025<=0]=0
ICy975 = med_y+1.96*sqrt(var_y)

#--- Accumulated and IC --- #
accum_y=cumsum(y)
accum_med_Y=cumsum(med_y)
accum_s2=cumsum(var_y)                     
accum_IC025=accum_med_Y-1.96*sqrt(accum_s2)   
accum_IC975=accum_med_Y+1.96*sqrt(accum_s2)   

#--------- Graphic: Mean and IC ----------------#
plot(dia,y,ylim=c(0,max(y,ICy975)),xlab="Days" ,ylab="Number of New Deaths",main = " ",lwd=1,pch=8,cex=0.8)
lines(dia,med_y,col="black",lwd=2)
lines(dia,ICy025,col="red",lty=2,lwd=2)
lines(dia,ICy975,col="red",lty=2,lwd=2)
legend('topleft', legend = c("Data", "CI (95%)", "Mean"),pch=c(8,NA,NA),lty=c(NA,2,1),
        col=c("black","red","black"),cex=1,border=c(NA,NA,NA),fill = c(NA,NA,NA),lwd=c(1,2,2))

x11()
#--------- Graphic Accumlated Mean and IC ----#
plot(dia,accum_y,ylim=c(0,max(y,accum_IC975)),xlab="Days" ,ylab="Accumulated Deaths",main = " ",lwd=1,pch=8,cex=0.8)
lines(dia,accum_med_Y,col="black",lwd=2)
lines(dia,accum_IC025,col="red",lty=2,lwd=2)
lines(dia,accum_IC975,col="red",lty=2,lwd=2)
legend('topleft', legend = c("Data", "CI (95%)", "Accum. Mean"),pch=c(8,NA,NA),lty=c(NA,2,1),
        col=c("black","red","black"),cex=1,border=c(NA,NA,NA),fill = c(NA,NA,NA),lwd=c(1,2,2))


#--------- Graphic: Z(t) Residual analysis -----------------------------#
par(mfrow = c(2,3))
ts.plot(Z, gpars=list(xlab="Dayes", ylab="Z", lty=c(1))) 
acf(Z)
pacf(Z)
ts.plot(Z2, gpars=list(xlab="Dayes", ylab=expression(Z^2), lty=c(1)))
acf(Z2)
pacf(Z2)
#-----------------------------------------------------------------------#
x11()
#--------- Graphic: W(t) Residual analysis -----------------------------#
par(mfrow = c(2,3))
ts.plot(W, gpars=list(xlab="Dayes", ylab="W", lty=c(1))) 
acf(W)
pacf(W)
ts.plot(W2, gpars=list(xlab="Dayes", ylab=expression(W^2), lty=c(1)))
acf(W2)
pacf(W2)
#-----------------------------------------------------------------------#

x11()
par(mfrow = c(2,2))
plot(dia,Ws,xlab="Days" ,ylab="Standardized Residue W",main = " ",lwd=1,pch=8,cex=1)
acf(Ws)
hist(Ws,nclass=30, main=" ", xlab="Residue" ,prob=T)
lines(seq(-1+min(Ws),max(Ws),len=1000),dnorm(seq(-1+min(Ws),max(Ws),len=1000)),
      lwd=2,col=2)
qqnorm(Ws, main = "Normal Q-Q Plot",
       xlab = "Theoretical Quantiles", ylab = "Residue Quantiles")
qqline(Ws, distribution = qnorm, probs = c(0.25, 0.75), qtype = 7)
# ---------------------------------------------------------------------#

#-- Save Generated Chain ----#
#write.table(theta_bayes, file = paste('Theta_',mod_name,'_',Country,'.txt'), sep = " ",row.names=F,col.names=F)

#write.table(Res, file = paste('Res_',mod_name,'_',Country,'.txt'), sep = " ",row.names=F,col.names=F)
#write.table(Res_stand, file = paste('Res_stand_',mod_name,'_',Country,'.txt'), sep = " ",row.names=F,col.names=F)


# --- Graphics For Chain -------#

#############  Graphics #############################################################
x11()
par(mfrow = c(1,3))
hist(theta_bayes[,1],nclass=30, main=" ", xlab=expression(theta[1]) )
abline(v = mean(theta_bayes[,1]),col=1,lwd=2,lty=2 )
hist(theta_bayes[,2],nclass=30, main=" ", xlab=expression(theta[2]))
abline(v = mean(theta_bayes[,2]),col=1,lwd=2,lty=2  )
hist(theta_bayes[,3],nclass=30, main=" ", xlab=expression(theta[3]))
abline(v = mean(theta_bayes[,3]),col=1,lwd=2,lty=2  )

x11()
par(mfrow = c(1,3))
plot(theta_bayes[,1], type = "l", xlab="iterations" ,ylab=expression(theta[1]), main = " ")
##abline(h =  mean(chain[-(1:burnIn),1]), col="red" )
plot(theta_bayes[,2], type = "l", xlab="iterations" ,ylab=expression(theta[2]) , main = " ")
##abline(h =  mean(chain[-(1:burnIn),2]), col="red" )
plot(theta_bayes[,3], type = "l", xlab="iterations" ,ylab=expression(theta[3]) , main = " ")
##abline(h =  mean(chain[-(1:burnIn),3]), col="red" )

############## Graphics - Modelo Var #########################################
x11()
par(mfrow = c(2,2))
hist(theta_bayes[,4],nclass=30, , main=" ", xlab=expression(alpha[1]) )
abline(v = mean(theta_bayes[,4]),col=1,lwd=2,lty=2 )
hist(theta_bayes[,5],nclass=30, , main=" ", xlab=expression(alpha[2]) )
abline(v = mean(theta_bayes[,5]),col=1,lwd=2,lty=2  )
plot(theta_bayes[,4], type = "l", xlab="iterations" ,ylab=expression(alpha[1]) , main = " ", )
#abline(h =  mean(chain[-(1:burnIn),5]), col="red" )
plot(theta_bayes[,5], type = "l", xlab="iterations" ,ylab=expression(alpha[2]) , main = " ", )
#abline(h =  mean(chain[-(1:burnIn),6]), col="red" )


## Graphics for Rational Function ##

#############  Graphics #############################################################
x11()
par(mfrow = c(2,2))
hist(theta_bayes[,1],nclass=30, main=" ", xlab=expression(theta[1]) )
abline(v = mean(theta_bayes[,1]),col=1,lwd=2,lty=2 )
hist(theta_bayes[,2],nclass=30, main=" ", xlab=expression(theta[2]))
abline(v = mean(theta_bayes[,2]),col=1,lwd=2,lty=2  )
hist(theta_bayes[,3],nclass=30, main=" ", xlab=expression(theta[3]))
abline(v = mean(theta_bayes[,3]),col=1,lwd=2,lty=2  )
hist(theta_bayes[,4],nclass=30, main=" ", xlab=expression(theta[4]))
abline(v = mean(theta_bayes[,4]),col=1,lwd=2,lty=2  )

x11()
par(mfrow = c(2,2))
plot(theta_bayes[,1], type = "l", xlab="iterations" ,ylab=expression(theta[1]), main = " ")
##abline(h =  mean(chain[-(1:burnIn),1]), col="red" )
plot(theta_bayes[,2], type = "l", xlab="iterations" ,ylab=expression(theta[2]) , main = " ")
##abline(h =  mean(chain[-(1:burnIn),2]), col="red" )
plot(theta_bayes[,3], type = "l", xlab="iterations" ,ylab=expression(theta[3]) , main = " ")
##abline(h =  mean(chain[-(1:burnIn),3]), col="red" )
plot(theta_bayes[,4], type = "l", xlab="iterations" ,ylab=expression(theta[4]) , main = " ")
##abline(h =  mean(chain[-(1:burnIn),3]), col="red" )


############## Graphics - Modelo Var #########################################
x11()
par(mfrow = c(2,2))
hist(theta_bayes[,5],nclass=30, , main=" ", xlab=expression(alpha[1]) )
abline(v = mean(theta_bayes[,5]),col=1,lwd=2,lty=2 )
hist(theta_bayes[,6],nclass=30, , main=" ", xlab=expression(alpha[2]) )
abline(v = mean(theta_bayes[,6]),col=1,lwd=2,lty=2  )
plot(theta_bayes[,5], type = "l", xlab="iterations" ,ylab=expression(alpha[1]) , main = " ", )
#abline(h =  mean(chain[-(1:burnIn),5]), col="red" )
plot(theta_bayes[,6], type = "l", xlab="iterations" ,ylab=expression(alpha[2]) , main = " ", )
#abline(h =  mean(chain[-(1:burnIn),6]), col="red" )


##############  Forecast    ##############################################

h_prev=100
Y_Obs=c()

##------- Model -----------###
#1. Polinomial
funcao_mu_prev=Poli_mu_prev
#

#2. Log-Normal
funcao_mu_prev=LogNorm_mu_prev
#

#3. Gaussian
funcao_mu_prev=Gauss_mu_prev
#

#4. Gamma
funcao_mu_prev=Gamma_mu_prev
#

#5. Exp. General.
funcao_mu_prev=ExpGen_mu_prev
#

#6. Alpha.
funcao_mu_prev=Alpha_mu_prev
#

#7. Logistico.
funcao_mu_prev=Logistico_mu_prev
#

##--------------------------###

dia_prev=seq(length(dia)+1, length(dia)+h_prev)

med_prev=funcao_mu_prev(theta_ch_bayes)
var_prev=funcao_sig_prev(theta_ch_bayes)

ICprev025 =med_prev-1.96*sqrt(var_prev)
ICprev025[ICprev025<=0]=0
ICprev975 = med_prev+1.96*sqrt(var_prev)

dia_total=c(dia,dia_prev)
med_total=c(med_y,med_prev)
var_total=c(var_y,var_prev)
IC025_total=c(ICy025,ICprev025)
IC975_total=c(ICy975,ICprev975)

#--- Forecast - Accumulated and IC --- #
accum_y=cumsum(y)
accum_med_total=cumsum(med_total)
accum_s2_total=cumsum(var_total)                     
accum_IC025_total=accum_med_total-1.96*sqrt(accum_s2_total)  
accum_IC975_total=accum_med_total+1.96*sqrt(accum_s2_total)   


#--------- Graphic: Forecast da Mean and CI ----------------#
plot(dia,y,xlim=c(0,length(dia_total)),ylim=c(0,max(y,IC975_total)),xlab="Days" ,ylab="Number of New Deaths",main = " ",lwd=1,pch=8,cex=0.8)
lines(dia_total,med_total,col="black",lwd=2)
lines(dia_total,IC025_total,col="red",lty=2,lwd=2)
lines(dia_total,IC975_total,col="red",lty=2,lwd=2)
legend('topright', legend = c("Data", "CI (95%)", "Mean"),pch=c(8,NA,NA),lty=c(NA,2,1),
        col=c("black","red","black"),cex=1,border=c(NA,NA,NA),fill = c(NA,NA,NA),lwd=c(1,2,2))

x11()
#--------- Graphic Forecast - Accumlated Mean and CI ----#
plot(dia,accum_y,xlim=c(0,length(dia_total)),ylim=c(0,max(y,accum_IC975_total)),xlab="Days" ,ylab="Accumulated Deaths",main = " ",lwd=1,pch=8,cex=0.8)
lines(dia_total,accum_med_total,col="black",lwd=2)
lines(dia_total,accum_IC025_total,col="red",lty=2,lwd=2)
lines(dia_total,accum_IC975_total,col="red",lty=2,lwd=2)
legend('bottomright', legend = c("Data", "CI (95%)", "Accum. Mean"),pch=c(8,NA,NA),lty=c(NA,2,1),
        col=c("black","red","black"),cex=1,border=c(NA,NA,NA),fill = c(NA,NA,NA),lwd=c(1,2,2))


#---- Ppeak Forecast-------#

prev.dia_peak=dia_total[which(med_total==max(med_total))]
real.dia_peak=dia[which(y==max(y))]

y.real.peak=y[real.dia_peak]
y.prev.dia.peak=y[prev.dia_peak]
prev.peak=med_total[prev.dia_peak]
prev.real.dia.peak=med_total[real.dia_peak]

c(prev.dia_peak, prev.peak, y.prev.dia.peak,IC025_total[prev.dia_peak],IC975_total[prev.dia_peak])
c(real.dia_peak,prev.real.dia.peak,y.real.peak,IC025_total[real.dia_peak],IC975_total[real.dia_peak] )

c(prev.dia_peak,scale*prev.peak, scale*y.prev.dia.peak,scale*c(IC025_total[prev.dia_peak],IC975_total[prev.dia_peak]))
c(real.dia_peak,scale*prev.real.dia.peak,scale*y.real.peak,scale*c(IC025_total[real.dia_peak],IC975_total[real.dia_peak]) )

# Scale
c(scale)

###----- Forecast the end of the pandemic ------###

h_prev=1000
dia_prev=seq(length(dia)+1, length(dia)+h_prev)
med_prev=funcao_mu_prev(theta_ch_bayes)
med_total=c(med_y,med_prev)
accum_med_total=cumsum(med_total)
deaths_total=accum_med_total[length(accum_med_total)]

End_pandemic_099=length(accum_med_total[accum_med_total<=0.99*deaths_total])
End_pandemic_097=length(accum_med_total[accum_med_total<=0.97*deaths_total])

c(End_pandemic_097,End_pandemic_099)

#----  Forecast Error ------#

h_prev=20
Y_future_values=COVID19[(length(y)+1):(length(y)+h_prev),2]
Y_Obs=Y_future_values

dia_prev=seq(length(dia)+1, length(dia)+h_prev)
var_prev=funcao_sig_prev(theta_ch_bayes)
med_prev=funcao_mu_prev(theta_ch_bayes) 

ICprev025 =med_prev-1.96*sqrt(var_prev)
ICprev025[ICprev025<=0]=0
ICprev975 = med_prev+1.96*sqrt(var_prev)

#if (Country=="Spain") Y_Obs[15]=scale*med_prev[15]
if (Country=="Spain") {Y_Obs[3]=Y_Obs[4]=Y_Obs[10]=mean(scale*med_prev)}



# MSE
MSE=(Y_Obs-scale*med_prev)^2

# MAE
MAE=abs(Y_Obs-scale*med_prev)

# MAPE
MAPE=100*(abs((Y_Obs-scale*med_prev)/Y_Obs))

# MAAPE
MAAPE=100*(atan(abs((Y_Obs-scale*med_prev)/Y_Obs)))


Error=cbind(MSE,MAE,MAPE,MAAPE)
Mean_Error= apply(Error,2,mean)
Forecst_error=c(sqrt(Mean_Error[1]),Mean_Error[2:4] )

Forecst_error 

#--------- Graphic: Forecast da Mean and CI ----------------#
plot(dia_prev,Y_Obs,ylim=c(min(Y_Obs,scale*ICprev025),max(Y_Obs,scale*ICprev975)),
     xlim=c(min(dia_prev),max(dia_prev)), xlab="Days" ,ylab="Number of New Deaths",main = " ",lwd=1,pch=8,cex=0.8)
lines(dia_prev,scale*med_prev,col="black",lwd=2)
lines(dia_prev,scale*ICprev025,col="red",lty=2,lwd=3)
lines(dia_prev,scale*ICprev975,col="red",lty=2,lwd=3)
legend('topleft', legend = c("Data", "CI (95%)", "Forecast"),pch=c(8,NA,NA),lty=c(NA,2,1),
        col=c("black","red","black"),cex=1,border=c(NA,NA,NA),fill = c(NA,NA,NA),lwd=c(1,3,2))
#-----------------------------------------------------------#


#----- Graphis - Forecast X Observed -----###
plot(Y_Obs,scale*med_prev,ylim=c(min(Y_Obs,scale*med_prev),max(Y_Obs,scale*med_prev)),
                          xlim=c(min(Y_Obs,scale*med_prev),max(Y_Obs,scale*med_prev)),
                          pch=8)
lines(Y_Obs,Y_Obs,ylim=c(min(Y_Obs,scale*med_prev),max(Y_Obs,scale*med_prev)),
                  xlim=c(min(Y_Obs,scale*med_prev),max(Y_Obs,scale*med_prev)),
                  col=1,lwd=2)
#-----------------------------------------------------

